@@ -46,12 +46,12 @@ class CodeBase(object):
     def complete(self):
         """ return one `Completion` instance or None. """
         # If there is one completion, return that.
-        completions = list(self.get_completions(True))
+        completions = list(self.get_completions())
 
         # Return the common prefix.
         return _commonprefix([ c.suffix for c in completions ])
 
-    def get_completions(self, recursive=False):
+    def get_completions(self):
         """ Yield `Completion` instances. """
         if False:
             yield
@@ -73,7 +73,7 @@ class Code(CodeBase):
     :attr document: :class:`~prompt_toolkit.line.Document`
     """
     #: The pygments Lexer class to use.
-    lexer_cls = None
+    lexer = None
 
     def __init__(self, document):
         super(Code, self).__init__(document)
@@ -82,8 +82,8 @@ class Code(CodeBase):
     @property
     def _lexer(self):
         """ Return lexer instance. """
-        if self.lexer_cls:
-            return self.lexer_cls(
+        if self.lexer:
+            return self.lexer(
                     stripnl=False,
                     stripall=False,
                     ensurenl=False)
