@@ -4,29 +4,29 @@ Useful shortcuts.
 from __future__ import unicode_literals
 
 from .. import CommandLine, AbortAction
-from ..prompt import Prompt
+from ..prompt import Prompt, PasswordProcessor
 from ..inputstream_handler import EmacsInputStreamHandler
 from ..line import Line
 
 
-def get_input(message, raise_exception_on_abort=False, multiline=False):
+def get_input(message, raise_exception_on_abort=False, multiline=False, is_password=False):
     """
     Replacement for `raw_input`.
     Ask for input, return the answer.
     This returns `None` when Ctrl-D was pressed.
     """
     class CustomPrompt(Prompt):
-        default_prompt_text = message
-
+        prompt_text = message
+        input_processors = ([PasswordProcessor()] if is_password else [])
 
     class CustomLine(Line):
         is_multiline = multiline
 
 
     class CLI(CommandLine):
-        prompt_cls = CustomPrompt
-        inputstream_handler_cls = EmacsInputStreamHandler
-        line_cls = CustomLine
+        prompt_factory = CustomPrompt
+        inputstream_handler_factory = EmacsInputStreamHandler
+        line_factory = CustomLine
 
 
     cli = CLI()
