@@ -207,6 +207,13 @@ class Document(object):
                 index += 1
             return index
 
+    def find_all(self, sub):
+        """
+        Find all occurances of the substring. Return a list of absolute
+        positions in the document.
+        """
+        return [a.start() for a in re.finditer(re.escape(sub), self.text)]
+
     def find_backwards(self, sub, in_current_line=False):
         """
         Find `text` before the cursor, return position relative to the cursor
@@ -222,6 +229,9 @@ class Document(object):
         if index >= 0:
             return index - len(before_cursor)
 
+    def get_word_before_cursor(self):
+        return self.text_before_cursor[self.find_start_of_previous_word():]
+
     def find_start_of_previous_word(self):
         """
         Return an index relative to the cursor position pointing to the start
