@@ -37,7 +37,7 @@ class InputStreamHandler(object):
         #: The name of the last previous public function call.
         self._last_call = None
 
-        self.__arg_count = None
+        self._arg_count = None
 
     @property
     def _arg_count(self):
@@ -225,6 +225,13 @@ class InputStreamHandler(object):
         """
         Autocomplete.
         """
+        self.do_traditional_complete()
+
+    def do_traditional_complete(self):
+        """
+        Traditional tab-completion, where the first tab completes the common
+        suffix and the second tab lists all the completions.
+        """
         if self._second_tab:
             self._line.list_completions()
             self._second_tab = False
