@@ -13,6 +13,8 @@ from .prompt import Prompt
 from .render_context import RenderContext
 from .history import History
 
+from pygments.token import Token
+
 import os
 import tempfile
 import subprocess
@@ -24,6 +26,7 @@ __all__ = (
         'Exit',
         'ReturnInput',
         'Abort',
+        'ClearScreen',
 )
 
 class Exit(Exception):
@@ -42,6 +45,16 @@ class Abort(Exception):
         self.render_context = render_context
 
 
+class ClearScreen(Exception):
+    pass
+
+
+class ListCompletions(Exception):
+    def __init__(self, render_context, completions):
+        self.render_context = render_context
+        self.completions = completions
+
+
 class ClipboardDataType(object):
     """
     Depending on how data has been copied, it can be pasted differently.
@@ -123,6 +136,10 @@ class _IncrementalSearchState(object):
         self.original_working_index = original_working_index
         self.original_cursor_position = original_cursor_position
 
+        #: From this character index, we didn't found any more matches.
+        #: This flag is updated every time we search for a new string.
+        self.no_match_from_index = None
+
 
 class Line(object):
     """
@@ -131,8 +148,7 @@ class Line(object):
     also implements the history, undo stack, reverse search and the completion
     state.
 
-    :attr code_cls: :class:`~prompt_toolkit.code.CodeBase` class.
-    :attr prompt_cls: :class:`~prompt_toolkit.prompt.PromptBase` class.
+    :attr code_factory: :class:`~prompt_toolkit.code.CodeBase` class.
     :attr history: :class:`~prompt_toolkit.history.History` instance.
     """
     #: Boolean to indicate whether we should consider this line a multiline input.
@@ -140,13 +156,14 @@ class Line(object):
     #: (Instead of accepting the input.)
     is_multiline = False
 
-    def __init__(self, renderer=None, code_cls=Code, prompt_cls=Prompt, history_cls=History):
-        self.renderer = renderer
-        self.code_cls = code_cls
-        self.prompt_cls = prompt_cls
+    #: Suffix to be appended to the tempfile for the 'open in editor' function.
+    tempfile_suffix = ''
+
+    def __init__(self, code_factory=Code, history_factory=History):
+        self.code_factory = code_factory
 
         #: The command line history.
-        self._history = history_cls()
+        self._history = history_factory()
 
         self._clipboard = ClipboardData()
 
@@ -169,7 +186,7 @@ class Line(object):
         self.isearch_state = None
 
         # State of complete browser
-        self.complete_state = None
+        self.complete_state = None # For interactive completion through Ctrl-N/Ctrl-P.
 
         # Undo stack
         self._undo_stack = [] # Stack of (text, cursor_position)
@@ -297,7 +314,7 @@ class Line(object):
         # TODO: this can be cached as long self.text does not change.
         return Document(self.text, self.cursor_position)
 
-    def set_arg_prompt(self, arg):
+    def set_arg_prompt(self, arg): # XXX: Just make a property of this???
         """
         Called from the `InputStreamHandler` to set a "(arg: x)"-like prompt.
         (Both in Vi and Emacs-mode we have a way to repeat line operations.
@@ -414,11 +431,16 @@ class Line(object):
     @_to_mode(LineMode.NORMAL, LineMode.INCREMENTAL_SEARCH)
     def delete_character_before_cursor(self, count=1): # TODO: unittest return type
         """ Delete character before cursor, return deleted character. """
-        assert count > 0
+        assert count >= 0
         deleted = ''
 
         if self.mode == LineMode.INCREMENTAL_SEARCH:
             self.isearch_state.isearch_text = self.isearch_state.isearch_text[:-count]
+
+            # When the `no_match_from_index` is after the character that we deleted. Remove this mark.
+            if (self.isearch_state.no_match_from_index is not None and
+                    self.isearch_state.no_match_from_index >= len(self.isearch_state.isearch_text)):
+                self.isearch_state.no_match_from_index = None
         else:
             if self.cursor_position > 0:
                 deleted = self.text[self.cursor_position - count:self.cursor_position]
@@ -573,7 +595,7 @@ class Line(object):
         """
         Create `Code` instance from the current input.
         """
-        return self.code_cls(self.document)
+        return self.code_factory(self.document)
 
     @_to_mode(LineMode.NORMAL)
     def list_completions(self):
@@ -582,8 +604,8 @@ class Line(object):
         """
         results = list(self.create_code_obj().get_completions())
 
-        if results and self.renderer:
-            self.renderer.render_completions(results)
+        if results:
+            raise ListCompletions(self.get_render_context(), results)
 
     @_to_mode(LineMode.NORMAL)
     def complete(self): # TODO: rename to complete_common
@@ -673,30 +695,28 @@ class Line(object):
         """
         Return a `RenderContext` object, to pass the current state to the renderer.
         """
+        highlighted_characters = { }
+        code = self.create_code_obj()
+
         if self.mode == LineMode.INCREMENTAL_SEARCH:
-            # In case of reverse search, render reverse search prompt.
-            code = self.code_cls(self.document)
+            # In case of reverse search, highlight all matches.
+            for index in self.document.find_all(self.isearch_state.isearch_text):
+                if index == self.cursor_position:
+                    token = Token.IncrementalSearchMatch.Current
+                else:
+                    token = Token.IncrementalSearchMatch
 
-            if self.document.has_match_at_current_position(self.isearch_state.isearch_text):
-                highlight_regions = [
-                        (self.document.translate_index_to_position(self.cursor_position),
-                        self.document.translate_index_to_position(self.cursor_position + len(self.isearch_state.isearch_text))) ]
-            else:
-                highlight_regions = [ ]
-
-        else:
-            code = self.create_code_obj()
-            highlight_regions = [ ]
+                highlighted_characters.update({
+                        x: token for x in range(index, index + len(self.isearch_state.isearch_text)) })
 
         # Complete state
-        prompt = self.prompt_cls(self, code)
         if self.mode == LineMode.COMPLETE and not _abort and not _accept:
             complete_state = self.complete_state
         else:
             complete_state = None
 
         # Create prompt instance.
-        return RenderContext(prompt, code, highlight_regions=highlight_regions,
+        return RenderContext(self, code, highlighted_characters=highlighted_characters,
                         complete_state=complete_state,
                         abort=_abort, accept=_accept,
                         validation_error=self.validation_error)
@@ -751,8 +771,13 @@ class Line(object):
         if self.mode == LineMode.INCREMENTAL_SEARCH:
             self.isearch_state.isearch_text += data
 
+
             if not self.document.has_match_at_current_position(self.isearch_state.isearch_text):
-                self.search_next(self.isearch_state.isearch_direction)
+                found = self.search_next(self.isearch_state.isearch_direction)
+
+                # When this suffix is not found, remember that in `no_match_from_index`.
+                if not found and self.isearch_state.no_match_from_index is None:
+                    self.isearch_state.no_match_from_index = len(self.isearch_state.isearch_text) - 1
         else:
             # In insert/text mode.
             if overwrite:
@@ -887,9 +912,14 @@ class Line(object):
 
     @_to_mode(LineMode.NORMAL, LineMode.INCREMENTAL_SEARCH)
     def search_next(self, direction):
+        """
+        Search for the next string.
+        :returns: (bool) True if something was found.
+        """
         if not (self.mode == LineMode.INCREMENTAL_SEARCH and self.isearch_state.isearch_text):
             return
 
+        found = False
         self.isearch_state.isearch_direction = direction
 
         isearch_text = self.isearch_state.isearch_text
@@ -900,6 +930,7 @@ class Line(object):
 
             if new_index is not None:
                 self.cursor_position += new_index
+                found = True
             else:
                 # No match, go back in the history.
                 for i in range(self._working_index - 1, -1, -1):
@@ -908,6 +939,8 @@ class Line(object):
                     if new_index is not None:
                         self._working_index = i
                         self.cursor_position = len(self._working_lines[i]) + new_index
+                        self.isearch_state.no_match_from_index = None
+                        found = True
                         break
         else:
             # Try find at the current input.
@@ -915,6 +948,7 @@ class Line(object):
 
             if new_index is not None:
                 self.cursor_position += new_index
+                found = True
             else:
                 # No match, go forward in the history.
                 for i in range(self._working_index + 1, len(self._working_lines)):
@@ -923,7 +957,14 @@ class Line(object):
                     if new_index is not None:
                         self._working_index = i
                         self.cursor_position = new_index
+                        self.isearch_state.no_match_from_index = None
+                        found = True
                         break
+                else:
+                    # If no break: we didn't found a match.
+                    found = False
+
+        return found
 
     def exit_isearch(self, restore_original_line=False):
         """
@@ -939,16 +980,15 @@ class Line(object):
     @_to_mode(LineMode.NORMAL)
     def clear(self):
         """
-        Clear renderer screen, usually as a result of Ctrl-L.
+        Clear screen, usually as a result of Ctrl-L.
         """
-        if self.renderer:
-            self.renderer.clear()
+        raise ClearScreen()
 
     @_to_mode(LineMode.NORMAL)
     def open_in_editor(self):
         """ Open code in editor. """
         # Write to temporary file
-        descriptor, filename = tempfile.mkstemp()
+        descriptor, filename = tempfile.mkstemp(self.tempfile_suffix)
         os.write(descriptor, self.text.encode('utf-8'))
         os.close(descriptor)
 
@@ -982,6 +1022,10 @@ class Line(object):
 
         for e in editors:
             if e:
-                if os.path.exists(e):
+                try:
                     subprocess.call([e, filename])
                     return
+
+                except OSError:
+                    # Executable does not exist, try the next one.
+                    pass
